from http import HTTPStatus
from typing import Any, Dict, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.v1_check_permission_request import V1CheckPermissionRequest
from ...models.v1_check_permission_response import V1CheckPermissionResponse
from ...types import Response


def _get_kwargs(
    *,
    json_body: V1CheckPermissionRequest,
) -> Dict[str, Any]:
    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": "/v1/permissions/check",
        "json": json_json_body,
    }


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[V1CheckPermissionResponse]:
    if response.status_code == HTTPStatus.OK:
        response_200 = V1CheckPermissionResponse.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[V1CheckPermissionResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    json_body: V1CheckPermissionRequest,
) -> Response[V1CheckPermissionResponse]:
    """CheckPermission determines for a given resource whether a subject computes
    to having a permission or is a direct member of a particular relation.

    Args:
        json_body (V1CheckPermissionRequest): CheckPermissionRequest issues a check on whether a
            subject has a permission
            or is a member of a relation, on a specific resource.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[V1CheckPermissionResponse]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: Union[AuthenticatedClient, Client],
    json_body: V1CheckPermissionRequest,
) -> Optional[V1CheckPermissionResponse]:
    """CheckPermission determines for a given resource whether a subject computes
    to having a permission or is a direct member of a particular relation.

    Args:
        json_body (V1CheckPermissionRequest): CheckPermissionRequest issues a check on whether a
            subject has a permission
            or is a member of a relation, on a specific resource.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        V1CheckPermissionResponse
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
    ).parsed


async def asyncio_detailed(
    *,
    client: Union[AuthenticatedClient, Client],
    json_body: V1CheckPermissionRequest,
) -> Response[V1CheckPermissionResponse]:
    """CheckPermission determines for a given resource whether a subject computes
    to having a permission or is a direct member of a particular relation.

    Args:
        json_body (V1CheckPermissionRequest): CheckPermissionRequest issues a check on whether a
            subject has a permission
            or is a member of a relation, on a specific resource.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[V1CheckPermissionResponse]
    """

    kwargs = _get_kwargs(
        json_body=json_body,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: Union[AuthenticatedClient, Client],
    json_body: V1CheckPermissionRequest,
) -> Optional[V1CheckPermissionResponse]:
    """CheckPermission determines for a given resource whether a subject computes
    to having a permission or is a direct member of a particular relation.

    Args:
        json_body (V1CheckPermissionRequest): CheckPermissionRequest issues a check on whether a
            subject has a permission
            or is a member of a relation, on a specific resource.

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        V1CheckPermissionResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            json_body=json_body,
        )
    ).parsed
