from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.v1_bulk_check_permission_request_item import V1BulkCheckPermissionRequestItem
    from ..models.v1_consistency import V1Consistency


T = TypeVar("T", bound="V1BulkCheckPermissionRequest")


@_attrs_define
class V1BulkCheckPermissionRequest:
    """
    Attributes:
        consistency (Union[Unset, V1Consistency]): Consistency will define how a request is handled by the backend.
            By defining a consistency requirement, and a token at which those
            requirements should be applied, where applicable.
        items (Union[Unset, List['V1BulkCheckPermissionRequestItem']]):
    """

    consistency: Union[Unset, "V1Consistency"] = UNSET
    items: Union[Unset, List["V1BulkCheckPermissionRequestItem"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        consistency: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.consistency, Unset):
            consistency = self.consistency.to_dict()

        items: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.items, Unset):
            items = []
            for items_item_data in self.items:
                items_item = items_item_data.to_dict()

                items.append(items_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if consistency is not UNSET:
            field_dict["consistency"] = consistency
        if items is not UNSET:
            field_dict["items"] = items

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.v1_bulk_check_permission_request_item import V1BulkCheckPermissionRequestItem
        from ..models.v1_consistency import V1Consistency

        d = src_dict.copy()
        _consistency = d.pop("consistency", UNSET)
        consistency: Union[Unset, V1Consistency]
        if isinstance(_consistency, Unset):
            consistency = UNSET
        else:
            consistency = V1Consistency.from_dict(_consistency)

        items = []
        _items = d.pop("items", UNSET)
        for items_item_data in _items or []:
            items_item = V1BulkCheckPermissionRequestItem.from_dict(items_item_data)

            items.append(items_item)

        v1_bulk_check_permission_request = cls(
            consistency=consistency,
            items=items,
        )

        v1_bulk_check_permission_request.additional_properties = d
        return v1_bulk_check_permission_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
